import numpy as np
from PIL import Image, ImageDraw, ImageFont

def _get_rect(x, y, width, height, angle):
    rect = np.array([(0, 0), (width, 0), (width, height), (0, height), (0, 0)])
    theta = (np.pi / 180.0) * angle
    R = np.array([[np.cos(theta), -np.sin(theta)],
                  [np.sin(theta), np.cos(theta)]])
    return np.dot(rect, R) + np.array([x, y])

def rect(pallete, x, y, w, h, angle=0, color=0):
    _rect = _get_rect(x, y, w, h, angle)
    img = Image.fromarray(pallete)
    draw = ImageDraw.Draw(img)
    draw.polygon([tuple(p) for p in _rect], fill=color)
    pallete[:, :, :] = np.asarray(img)


def line(pallete, x1, y1, x2, y2, color=0):
    img = Image.fromarray(pallete)
    draw = ImageDraw.Draw(img)
    draw.line((x1, y1, x2, y2), fill=color)
    pallete[:, :, :] = np.asarray(img)


def image(pallete, x, y, img):
    bg = Image.fromarray(pallete)
    if isinstance(img, str):
        _img = Image.open(img)
        bg.paste(_img, (x, y), _img)
    else:
        raise ValueError('unknown img type.')
    pallete[:, :, :] = np.asarray(bg)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['nummu']

package_data = \
{'': ['*']}

install_requires = \
['Pillow==5.0.0', 'array2gif==1.0.3', 'numpngw==0.0.6']

setup_kwargs = {
    'name': 'nummu',
    'version': '0.1.5',
    'description': 'An animated image maker.',
    'long_description': "# nummu (南無)\n\nAn animated image maker.\n\nNOTE: this project in WIP and thus interface might change a lot. Be cautious when using in production.\n\n|Image|Script|\n|-----|------|\n|![helloworld](https://github.com/soasme/nummu/raw/master/examples/helloworld.png)|[examples/helloworld.py](examples/helloworld.py)|\n|![line](https://github.com/soasme/nummu/raw/master/examples/line.gif)|[examples/line.py](examples/line.py)|\n|![rect](https://github.com/soasme/nummu/raw/master/examples/rect.gif)|[examples/rect.py](examples/rect.py)|\n|![bubblesort](https://gist.github.com/soasme/e3f1a210cc7e7750d304cb43b6aaad23/raw/d6e5687ce8492bd5865f4d7f38a5ec421c5d4c1b/bubblesort.png) Bubble Sort<br> ![heapsort](https://gist.github.com/soasme/e3f1a210cc7e7750d304cb43b6aaad23/raw/d6e5687ce8492bd5865f4d7f38a5ec421c5d4c1b/heapsort.png) Heap Sort<br> ![insertionsort](https://gist.github.com/soasme/e3f1a210cc7e7750d304cb43b6aaad23/raw/d6e5687ce8492bd5865f4d7f38a5ec421c5d4c1b/insertionsort.png) Insertion Sort<br> ![mergesort](https://gist.github.com/soasme/e3f1a210cc7e7750d304cb43b6aaad23/raw/d6e5687ce8492bd5865f4d7f38a5ec421c5d4c1b/mergesort.png) Merge Sort<br> ![quicksort](https://gist.github.com/soasme/e3f1a210cc7e7750d304cb43b6aaad23/raw/d6e5687ce8492bd5865f4d7f38a5ec421c5d4c1b/quicksort.png) Quick Sort<br> ![selectionsort](https://gist.github.com/soasme/e3f1a210cc7e7750d304cb43b6aaad23/raw/d6e5687ce8492bd5865f4d7f38a5ec421c5d4c1b/selectionsort.png) Selection Sort |[examples/sort.py](examples/sort.py)|\n|![image](https://github.com/soasme/nummu/raw/master/examples/ghost.gif)|[examples/image.py](examples/image.py)|\n\n## Install\n\n```\n$ pip install nummu\n```\n\n## Usage\n\nBy extending Nummu, you might want to implement a class having these methods: `init()`, `update(delta)`, `draw(palette)`. None of them are absolutely required.\n\n    class HelloWorld:\n\n        def init(self):\n            # Do initial work here\n            self.position = 0\n\n        def update(self, delta):\n            # Do some calculation before draw each frame.\n            # The delta is in millisecond unit.\n            #\n            # Don't forget to raise StopIteration somewhere!\n            #\n            self.position += delta\n            if self.position >= 100:\n                raise StopIteration\n\n        def draw(self, pallete):\n            # Pallete is simply a numpy.zeros instance.\n            # Overwrite any pixels as you want.\n            #\n            pallete[:, self.position, :] = 255\n\nBasic usage:\n\n    # import nummu\n    from nummu import Nummu\n\n    # set file size\n    nm = Nummu(100, 100)\n\n    # extend nummu\n    nm.extend(HelloWorld())\n\n    # export to gif\n    nm.export('helloworld.gif', delay=5)\n\n\nCheck the examples in project repo, and hopefully it might intrigue you some!\n\n## Develop\n\n```\n$ poetry develop\n$ poetry run python examples/helloworld.py\n```\n\n## Test\n\nCurrently nummu support running on 3.6, 3.7\n\n```\n$ pyenv install 3.6.5\n$ pyenv install 3.7.0b2\n$ pyenv local 3.6.5 3.7.0b2\n$ export PATH=$(pyenv root)/shims:$PATH\n$ tox\npy36 runtests: commands[0] | poetry develop\npy36 runtests: commands[0] | poetry run pytest\npy37 runtests: commands[0] | poetry develop\npy37 runtests: commands[0] | poetry run pytest\n  py36: commands succeeded\n  py37: commands succeeded\n  congratulations :)\n```\n\n## Credit\n\nThanks Pillow, numpy, numpngw and array2gif! Nummu stands on the shoulder of these giants!\n",
    'author': 'Ju Lin',
    'author_email': 'soasme@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import numpy as np
from array2gif import write_gif
from numpngw import write_apng

class Nummu:

    def __init__(self, height, weight):
        self.seq = []
        self.is_quit = False
        self.modules = []
        self.resources = {}
        self.height = height
        self.weight = weight

    def add(self, name, type, **meta):
        self.resources[name] = dict(type=type, **meta)

    def extend(self, mod):
        self.modules.append(mod)

    def quit(self):
        self.is_quit = True

    def _run_init(self):
        for mod in self.modules:
            if hasattr(mod, 'init'):
                mod.init()

    def _run_update(self, delta):
        for mod in self.modules:
            if hasattr(mod, 'update'):
                mod.update(delta)

    def _run_draw(self):
        palette = np.zeros((self.height, self.weight, 3), dtype=np.uint8)
        for mod in self.modules:
            if hasattr(mod, 'draw'):
                mod.draw(palette)
        self.seq.append(palette)

    def export(self, filename, delay, length=30000):
        self._run_init()
        progress = 0
        while not self.is_quit:
            try:
                self._run_update(delay)
                self._run_draw()
                progress += delay
                if progress >= length:
                    raise StopIteration
            except StopIteration:
                self.quit()
        if filename.endswith('.png'):
            write_apng(filename, self.seq, delay=delay, use_palette=True)
        elif filename.endswith('.gif'):
            # array2gif requires a transposed matrix compared to numpngw.
            seq = []
            for s in self.seq:
                seq.append(np.transpose(s, axes=[1, 0, 2]))
            write_gif(seq, filename, fps=int(1000/delay))
        else:
            raise NotImplementedError('Unsupported file format')
